#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "ser_drv.h"

#define BUFF_IN_BYTES	2048

char buf[BUFF_IN_BYTES];
int buf_len;
char atozch[26]="abcdefghijklmnopqrstuvwxyz";

void modem_send(SER_HANDLE ser, int tx_mode, int tx_power, int tx_bytes, int blk_no);
void time_delay(int sec);

int main(int argc, char *argv[])
{
	ser_port_t comport;
	SER_HANDLE com = &comport;
	int i,j,k;
	int TX_STATUS=0;
	int mode_byte[5]={38, 80, 122, 164, 248};
	int blk_byte_num[5]={0};	// for 5 modes, initialized to be 0
	int repeat_num;			// repeat certain number per mode
	int test_blk_num;
	int tx_mode, power_pec;
	int delay_seconds;
	int max_power, min_power;
	int max_mode, min_mode;

	if (argc<3 || argc > 9)
	{
		printf("Please input 2 to 8 arguments!\n\r");
		printf("The first is port name!\n\r");
		printf("The second is baud rate!\n\r");
		printf("The third is lowest transmission mode to test! (default 1)\n\r");
		printf("The fourth is highest transmission mode to test! (default 5)\n\r");
		printf("The fifth is minimum power level! (default -127)\n\r");
		printf("The sixth is maximum power level! (default 0)\n\r");
		printf("The seventh is total number of blocks to be tested! (>=5, default 50 blocks)\n\r");
		printf("The eighth is delay interval between transmissions (default 4 seconds)!\n\r");
		return 0;
	}
	else if (argc == 3)
	{
		min_mode = 1;
		max_mode = 5;
		min_power = -127;
		max_power = 0;
		test_blk_num = 50;
		delay_seconds = 15;
	}
	else if (argc == 4)
	{
		min_mode = atoi(argv[3]);
		max_mode = 5;
		min_power = -127;
		max_power = 0;
		test_blk_num = 50;
		delay_seconds = 15;
	}
	else if (argc ==5)
	{
		min_mode = atoi(argv[3]);
		max_mode = atoi(argv[4]);
		min_power = -127;
		max_power = 0;
		test_blk_num = 50;
		delay_seconds = 15;
	}
	else if (argc == 6)
	{
		min_mode = atoi(argv[3]);
		max_mode = atoi(argv[4]);
		min_power = atoi(argv[5]);
		max_power = 0;
		test_blk_num = 50;
		delay_seconds = 15;
	}
	else if (argc == 7)
	{
		min_mode = atoi(argv[3]);
		max_mode = atoi(argv[4]);
		min_power = atoi(argv[5]);
		max_power = atoi(argv[6]);
		test_blk_num = 50;
		delay_seconds = 15;
	}
	else if (argc == 8)
	{
		min_mode = atoi(argv[3]);
		max_mode = atoi(argv[4]);
		min_power = atoi(argv[5]);
		max_power = atoi(argv[6]);
		test_blk_num = atoi(argv[7]);
		delay_seconds = 15;
	}
	else if (argc == 9)
	{
		min_mode = atoi(argv[3]);
		max_mode = atoi(argv[4]);
		min_power = atoi(argv[5]);
		max_power = atoi(argv[6]);
		test_blk_num = atoi(argv[7]);
		delay_seconds = atoi(argv[8]);
	}

	strcpy(com->port, argv[1]);
	com->baud = atoi(argv[2]);
	com->timeout = 10000;	// ms

	if( ser_open(com) )
	{
		printf("Fail to open serial port %s", com->port);
		return 0;
	}
	else
	{
		printf("Start transmission!\r\n");
	}

	for(i=0;i<5;i++)
	{
		blk_byte_num[i]=mode_byte[i]*5-mode_byte[i]/2;	// now set each transmission with 5 blocks
	}

	if(test_blk_num<5)
	{
		repeat_num = 1;
	}
	else
	{
		repeat_num = test_blk_num/5;
	}

	ser_flush(com, 2); //flush both input and ouput buffer of serial port.

	buf_len = sprintf(buf, "$HHCRW,TXPWR,%d\r\n", -6);
	ser_write(com, buf, buf_len);
	buf_len = sprintf(buf, "$HHTXA,0,1,0,Start link quality test!\r\n");
	ser_write(com, buf, buf_len);
	
	for(power_pec = max_power;power_pec>=min_power;power_pec-=6)
	{
		for (i=min_mode;i<=max_mode;i++)
		{
			for (j=0;j<repeat_num;j++)
			{
				modem_send(com, i, power_pec, blk_byte_num[i-1], j);
				time_delay(delay_seconds);
			}
		}
	}

	buf_len = sprintf(buf, "$HHCRW,TXPWR,%d\r\n", -6);
	ser_write(com, buf, buf_len);
	buf_len = sprintf(buf, "$HHTXA,0,1,0,End link quality test!\r\n");
	ser_write(com, buf, buf_len);

	ser_close(com);

	return 0;
}


void modem_send(SER_HANDLE ser, int tx_mode, int tx_power, int tx_bytes, int j)
{
	int i, tmp_ind;
	unsigned int iseed;
	char * tmpadd;
	int tmplen;

	buf_len = sprintf(buf, "$HHCRW,TXPWR,%d\r\n", tx_power);
	ser_write(ser, buf, buf_len);

	printf("%s\n\r",buf);

	iseed = (unsigned int)time(NULL);
	srand (iseed);

	buf_len = sprintf(buf, "$HHTXA,0,%d,0,%04d%04d",tx_mode, tx_power, j);
	for (i=0;i<tx_bytes;i++)
	{
		tmp_ind = 26.0*(rand()/(RAND_MAX+1.0));
		buf[buf_len] = atozch[tmp_ind];
		buf_len++;
	}
	buf[buf_len] ='\r';
	buf_len++;
	buf[buf_len] ='\n';
	buf_len++;
	buf[buf_len] ='\0';

	ser_write(ser, buf, buf_len);
	printf("%s\n\r",buf);
}

void time_delay(int sec)
{
	clock_t delay = sec*CLOCKS_PER_SEC;
	clock_t start_t = clock();
	while((clock()-start_t)<delay)
	{}
}


