#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/select.h>
#include <termios.h>
#include "ser_drv.h"

int ser_open(SER_HANDLE ser)
{
	struct termios tio;	
	struct timeval timeout;
	int rv, bd;
	
	if( (ser->fd = open(ser->port, O_RDWR)) == -1)
		return (-1);		
	/*configure serial port*/
	/*select baud rate*/
	switch(ser->baud)
	{
	case 9600:
		bd = B9600;
		break;
	case 19200:
		bd = B19200;
		break;
	case 38400:
		bd = B38400;
		break;
	case 115200:
		bd = B115200;
		break;
	default:
		close(ser->port);
		return (-1);
	}
	memset(&tio,0,sizeof(tio));
	tio.c_iflag = 0;
	tio.c_oflag = 0;
	tio.c_cflag = CS8|CREAD|CLOCAL;
	tio.c_lflag = 0;
	tio.c_cc[VMIN] = 1;
	tio.c_cc[VTIME] = 5;
	rv = cfsetospeed(&tio, bd);  
	rv |= cfsetispeed(&tio, bd);	
	rv |= tcsetattr(ser->fd, TCSANOW, &tio); 
	if(rv)
	{
		close(ser->port);
		return (-1);
	}
	
	return (0);	
}
void ser_close(SER_HANDLE ser)
{	
	close(ser->port);
}
/*
 * Read from serial port
 * Input param	ser
 * Input param	buf: buffer to store the read out bytes
 * Return value: 
 *  -1: device error
 *   0: timeout event
 *   >0: returned bytes in des buffer   
 */  
int ser_read(SER_HANDLE ser, char *buf, int bytes)
{
	fd_set set;
  	struct timeval tout;
	int rv;

	if(ser->fd == -1)
		return (-1);
	
	FD_ZERO(&set); /* clear the set */
  	FD_SET(ser->fd, &set); /* add our file descriptor to the set */	
	tout.tv_sec = ser->timeout / 1000;
	tout.tv_usec = (ser->timeout - tout.tv_sec*1000) * 1000;

	if( (rv = select(ser->fd + 1, &set, NULL, NULL, &tout)) == -1)
		return (-2);
	else if(rv == 0)
	{
		/* a timeout occured */
		return (0);
	}
	else
	{
		/* there was data to read */
		return read( ser->fd, buf, bytes);
	}		
}	

/*
 * Write to serial port
 * Input param ser
 * Input param buf: buffer to be written to serial port
 * Input param bytes: 
 *  >0: bytes to be written
 *   0: treat buf as c string		
 * Return value: 
 *  -1: device error
 *  >0: bytes that has been written to serial port   
 */
int ser_write(SER_HANDLE ser, char *buf, int bytes)
{
	if(ser->fd == -1)
		return (-1);
	
	if(!bytes)
		bytes = strlen(buf);

	return (write(ser->fd, buf, bytes));
}
int ser_flush(SER_HANDLE ser, int flag)
{
	return	tcflush(ser->fd, flag);
}

int ser_timeout_set(SER_HANDLE ser, int tout)
{
	ser->timeout = tout;
	return 0;
}

