#ifndef _SER_DRV_H
#define _SER_DRV_H

typedef struct SER_PORT{
	int fd;
	char port[32];
	unsigned int baud;
	unsigned int timeout; //unit: ms
}ser_port_t;

typedef ser_port_t *SER_HANDLE;

/*
 * Open serial port
 * Return value:
 *   0: success
 *   -1: port error
 *   -2: fail to set timeout
 */
int ser_open(SER_HANDLE ser);

/* 
 * Close serial port
 */
void ser_close(SER_HANDLE ser);
/*
 * Read from serial port
 * Input param	ser
 * Input param	buf: buffer to store the read out bytes
 * Input param bytes: bytes to be read 
 * Return value: 
 *  -1: device error
 *   0: timeout event
 *   >0: returned bytes in des buffer   
 */  
int ser_read(SER_HANDLE ser, char *buf, int bytes);

/*
 * Write to serial port
 * Input param ser
 * Input param buf: buffer to be written to serial port
 * Input param bytes: 
 *  >0: bytes to be written
 *   0: treat buf as c string		
 * Return value: 
 *  -1: device error
 *  >0: bytes that has been written to serial port   
 */
int ser_write(SER_HANDLE ser, char *buf, int bytes);


/*
* set the timeout value
*/
int ser_timeout_set(SER_HANDLE ser, int tout);

#endif //_SER_DRV_H
